@@ -124,8 +124,6 @@ SEXP ocl_devices(SEXP platform, SEXP sDevType) {
     cl_device_id *did;
     cl_device_type dt = CL_DEVICE_TYPE_DEFAULT;
     const char *dts;
-    if (clGetDeviceIDs(pid, dt, 0, 0, &np) != CL_SUCCESS)
-	ocl_err("clGetDeviceIDs");
     if (TYPEOF(sDevType) != STRSXP || LENGTH(sDevType) != 1)
 	Rf_error("invalid device type - must be a character vector of length one");
     dts = CHAR(STRING_ELT(sDevType, 0));
@@ -141,6 +139,8 @@ SEXP ocl_devices(SEXP platform, SEXP sDevType) {
     }
     if (dt == CL_DEVICE_TYPE_DEFAULT && dts[0] != 'D' && dts[0] != 'd')
 	Rf_error("invalid device type - must be one of 'cpu', 'gpu', 'accelerator', 'default', 'all'.");
+    if (clGetDeviceIDs(pid, dt, 0, 0, &np) != CL_SUCCESS)
+	ocl_err("clGetDeviceIDs");
 
     res = Rf_allocVector(VECSXP, np);
     if (np > 0) {
@@ -273,6 +273,7 @@ SEXP ocl_ez_kernel(SEXP device, SEXP k_name, SEXP code, SEXP prec) {
 	Rf_setAttrib(sk, Rf_install("device"), device);
 	Rf_setAttrib(sk, Rf_install("precision"), prec);
 	Rf_setAttrib(sk, Rf_install("context"), sctx);
+	Rf_setAttrib(sk, Rf_install("name"), k_name);
 	UNPROTECT(2); /* sk + context */
 	return sk;
     }
@@ -385,14 +386,15 @@ SEXP ocl_call(SEXP args) {
     struct arg_chain *float_args = 0;
     ocl_call_context_t *occ;
     int on, an = 0, ftype = FT_DOUBLE, ftsize, ftres, async;
-    size_t global;
-    SEXP ker = CADR(args), olen, arg, res, octx;
+    SEXP ker = CADR(args), olen, arg, res, octx, dimVec;
     cl_kernel kernel = getKernel(ker);
     cl_context context;
     cl_command_queue commands;
     cl_device_id device_id = getDeviceID(getAttrib(ker, Rf_install("device")));
     cl_mem output;
     cl_int err;
+    size_t wdims[3] = {0, 0, 0};
+    int wdim = 1;
 
     if (clGetKernelInfo(kernel, CL_KERNEL_CONTEXT, sizeof(context), &context, NULL) != CL_SUCCESS || !context)
 	Rf_error("cannot obtain kernel context via clGetKernelInfo");
@@ -410,6 +412,19 @@ SEXP ocl_call(SEXP args) {
     if (ftype != FT_SINGLE) ftres = 0;
     args = CDR(args);
     async = (Rf_asInteger(CAR(args)) == 1) ? 0 : 1;  /* wait */
+    args = CDR(args);
+    dimVec = coerceVector(CAR(args), INTSXP);  /* dim */
+    wdim = LENGTH(dimVec);
+    if (wdim > 3)
+	Rf_error("OpenCL standard only supports up to three work item dimensions - use index vectors for higher dimensions");
+    if (wdim) {
+	int i; /* we don't use memcpy in case int and size_t are different */
+	for (i = 0; i < wdim; i++)
+	    wdims[i] = INTEGER(dimVec)[i];
+    }
+    if (wdim < 1 || wdims[0] < 1 || (wdim > 1 && wdims[1] < 1) || (wdim > 2 && wdims[2] < 1))
+	Rf_error("invalid dimensions - muse be a numeric vector with positive values");
+
     args = CDR(args);
     occ = (ocl_call_context_t*) calloc(1, sizeof(ocl_call_context_t));
     if (!occ) Rf_error("unable to allocate ocl_call context");
@@ -495,8 +510,7 @@ SEXP ocl_call(SEXP args) {
 	args = CDR(args);
     }
 
-    global = on;
-    if (clEnqueueNDRangeKernel(commands, kernel, 1, NULL, &global, NULL, 0, NULL, async ? &occ->event : NULL) != CL_SUCCESS)
+    if (clEnqueueNDRangeKernel(commands, kernel, wdim, NULL, wdims, NULL, 0, NULL, async ? &occ->event : NULL) != CL_SUCCESS)
 	Rf_error("Error during kernel execution");
 
     if (async) { /* asynchronous call -> get out and return the context */
